@@ -16,7 +16,10 @@ class BackboneSite(object):
             self._registry.remove(backbone_view_class)
 
     def get_urls(self):
-        from django.conf.urls.defaults import patterns, url
+        try:
+            from django.conf.urls import patterns, url
+        except ImportError:  # For backwards compatibility with Django <=1.3
+            from django.conf.urls.defaults import patterns, url
 
         urlpatterns = patterns('')
         for view_class in self._registry:
