@@ -5,10 +5,16 @@ from django.utils.translation import ugettext_lazy as _
 class Brand(models.Model):
     name = models.CharField(_('name'), max_length=255)
 
+    class Meta:
+        ordering = ('id',)
+
 
 class Category(models.Model):
     name = models.CharField(_('name'), max_length=255)
 
+    class Meta:
+        ordering = ('id',)
+
 
 class Product(models.Model):
     creation_date = models.DateTimeField(auto_now_add=True)
@@ -18,6 +24,7 @@ class Product(models.Model):
     is_hidden = models.BooleanField(default=False)
     price = models.DecimalField(max_digits=8, decimal_places=2)
     order = models.PositiveSmallIntegerField(default=0)
+    sku = models.CharField(max_length=255)
 
     class Meta:
         ordering = ('id',)
@@ -31,3 +38,7 @@ class Product(models.Model):
             return self.categories.all()[0].id
         else:
             return None
+
+
+class ExtendedProduct(Product):
+    description = models.CharField(max_length=255)
