@@ -1,13 +1,13 @@
 from decimal import Decimal
+import json
 
 from django.contrib.auth.models import User, Permission
 from django.core.urlresolvers import reverse
 from django.test import TestCase
-from django.utils import simplejson
 from django.utils.translation import ugettext as _
 
-from backbone.tests.models import Product, Brand, Category
-from backbone.tests.backbone_api import ProductBackboneView
+from backbone.tests.models import Product, Brand, Category, ExtendedProduct
+from backbone.tests.backbone_api import BrandBackboneView
 
 
 class TestHelper(TestCase):
@@ -15,10 +15,21 @@ class TestHelper(TestCase):
     def parseJsonResponse(self, response, status_code=200):
         self.assertEqual(response.status_code, status_code)
         self.assertEqual(response['Content-Type'], 'application/json')
-        data = simplejson.loads(response.content)
+        data = json.loads(response.content)
         return data
 
     def create_product(self, **kwargs):
+        defaults = {
+            'name': 'Test Product',
+            'price': '12.32',
+            'sku': '12345678'
+        }
+        if 'brand' not in kwargs:
+            defaults['brand'] = self.create_brand()
+        defaults.update(kwargs)
+        return Product.objects.create(**defaults)
+
+    def create_extended_product(self, **kwargs):
         defaults = {
             'name': 'Test Product',
             'price': '12.32'
@@ -26,7 +37,7 @@ class TestHelper(TestCase):
         if 'brand' not in kwargs:
             defaults['brand'] = self.create_brand()
         defaults.update(kwargs)
-        return Product.objects.create(**defaults)
+        return ExtendedProduct.objects.create(**defaults)
 
     def create_brand(self, **kwargs):
         defaults = {
@@ -69,7 +80,11 @@ class CollectionTests(TestHelper):
         self.assertEqual(len(data), 1)
         fields = data[0].keys()
 
-        self.assertEqual(set(['id'] + list(ProductBackboneView.display_fields)), set(fields))
+        expected_fields = [
+            'id', 'creation_date', 'name', 'brand', 'categories', 'price', 'order',
+            'is_priced_under_10', 'get_first_category_id', 'sku', 'custom2'
+        ]
+        self.assertEqual(set(expected_fields), set(fields))
         self.assertTrue('is_hidden' not in fields)
 
     def test_collection_view_foreign_key_is_returned_as_id(self):
@@ -112,6 +127,46 @@ class CollectionTests(TestHelper):
         response = self.client.delete(url)
         self.assertEqual(response.status_code, 403)
 
+    def test_collection_view_pagination(self):
+        # Brand is paginated 2 per page
+        p1 = self.create_brand()
+        p2 = self.create_brand()
+        p3 = self.create_brand()
+
+        url = reverse('backbone:tests_brand')
+
+        # First page
+        response = self.client.get(url, {'page': 1})
+        data = self.parseJsonResponse(response)
+        self.assertEqual(len(data), 2)
+        self.assertEqual(data[0]['id'], p1.id)
+        self.assertEqual(data[1]['id'], p2.id)
+
+        # Second Page
+        response = self.client.get(url, {'page': 2})
+        data = self.parseJsonResponse(response)
+        self.assertEqual(len(data), 1)
+        self.assertEqual(data[0]['id'], p3.id)
+
+    def test_collection_view_page_parameter_out_of_range_returns_error(self):
+        url = reverse('backbone:tests_brand')
+
+        response = self.client.get(url, {'page': 2})
+        self.assertEqual(response.status_code, 400)
+        self.assertEqual(response.content, _('Invalid `page` parameter: Out of range.'))
+
+    def test_collection_view_page_parameter_not_an_integer_returns_error(self):
+        url = reverse('backbone:tests_brand')
+
+        response = self.client.get(url, {'page': 'abcd'})
+        self.assertEqual(response.status_code, 400)
+        self.assertEqual(response.content, _('Invalid `page` parameter: Not a valid integer.'))
+
+    def test_collection_view_that_is_not_paginated_ignores_page_parameter(self):
+        url = reverse('backbone:tests_product')
+        response = self.client.get(url, {'page': 999})
+        self.assertEqual(response.status_code, 200)
+
 
 class DetailTests(TestHelper):
 
@@ -131,6 +186,10 @@ class DetailTests(TestHelper):
         self.assertEqual(data['order'], product.order)
         # Attribute on model
         self.assertEqual(data['is_priced_under_10'], True)
+        # Callable
+        self.assertEqual(data['sku'], '#: %s' % product.sku)
+        # Callable on admin class
+        self.assertEqual(data['custom2'], 'custom2: %s' % product.name)
         # Callable on model
         self.assertEqual(data['get_first_category_id'], category.id)
 
@@ -173,7 +232,7 @@ class AddTests(TestHelper):
         brand = self.create_brand()
         cat1 = self.create_category()
         cat2 = self.create_category()
-        data = simplejson.dumps({
+        data = json.dumps({
             'name': 'Test',
             'brand': brand.id,
             'categories': [cat1.id, cat2.id],
@@ -215,7 +274,7 @@ class AddTests(TestHelper):
         self.assertEqual(response.content, _('Unable to parse JSON request body.'))
 
     def test_post_request_on_product_collection_view_with_validation_errors_returns_error_list_as_json(self):
-        data = simplejson.dumps({
+        data = json.dumps({
             'name': '',
             'brand': '',
             'categories': [],
@@ -235,7 +294,7 @@ class AddTests(TestHelper):
         brand = self.create_brand()
         cat1 = self.create_category()
         cat2 = self.create_category()
-        data = simplejson.dumps({
+        data = json.dumps({
             'name': 'Test',
             'brand': brand.id,
             'categories': [cat1.id, cat2.id],
@@ -271,7 +330,7 @@ class AddTests(TestHelper):
     def test_post_request_on_product_collection_view_violating_field_specific_permission_returns_403(self):
         brand = self.create_brand()
         cat1 = self.create_category()
-        data = simplejson.dumps({
+        data = json.dumps({
             'name': 'NOTALLOWED',
             'brand': brand.id,
             'categories': [cat1.id],
@@ -284,7 +343,7 @@ class AddTests(TestHelper):
         self.assertEqual(response.content, _('You do not have permission to perform this action.'))
 
     def test_post_request_on_brand_collection_view_uses_custom_model_form(self):
-        data = simplejson.dumps({
+        data = json.dumps({
             'name': 'this should give an error',
         })
         url = reverse('backbone:tests_brand')
@@ -310,7 +369,7 @@ class UpdateTests(TestHelper):
         brand = self.create_brand()
         cat1 = self.create_category()
         cat2 = self.create_category()
-        data = simplejson.dumps({
+        data = json.dumps({
             'name': 'Test',
             'brand': brand.id,
             'categories': [cat1.id, cat2.id],
@@ -351,7 +410,7 @@ class UpdateTests(TestHelper):
 
     def test_put_request_on_product_detail_view_with_validation_errors_returns_error_list_as_json(self):
         product = self.create_product()
-        data = simplejson.dumps({
+        data = json.dumps({
             'name': '',
             'price': None,
             'order': '',
@@ -370,7 +429,7 @@ class UpdateTests(TestHelper):
         brand = self.create_brand()
         cat1 = self.create_category()
         cat2 = self.create_category()
-        data = simplejson.dumps({
+        data = json.dumps({
             'name': 'Test',
             'brand': brand.id,
             'categories': [cat1.id, cat2.id],
@@ -408,7 +467,7 @@ class UpdateTests(TestHelper):
         product = self.create_product()
         brand = self.create_brand()
         cat1 = self.create_category()
-        data = simplejson.dumps({
+        data = json.dumps({
             'name': 'NOTALLOWED',
             'brand': brand.id,
             'categories': [cat1.id],
@@ -422,7 +481,7 @@ class UpdateTests(TestHelper):
 
     def test_put_request_on_brand_collection_view_uses_custom_model_form(self):
         brand = self.create_brand()
-        data = simplejson.dumps({
+        data = json.dumps({
             'name': 'this should give an error',
         })
         url = reverse('backbone:tests_brand_detail', args=[brand.id])
@@ -476,3 +535,42 @@ class DeleteTests(TestHelper):
         self.assertEqual(response.status_code, 403)
         self.assertEqual(response.content, _('You do not have permission to perform this action.'))
         self.assertEqual(Brand.objects.count(), 1)
+
+
+class InheritanceTests(TestHelper):
+
+    def test_detail_view_returns_inherited_object_details(self):
+        ext_product = self.create_extended_product(price=9)
+        category = self.create_category()
+        ext_product.categories.add(category)
+
+        url = reverse('backbone:tests_extendedproduct_detail', args=[ext_product.id])
+        response = self.client.get(url)
+        data = self.parseJsonResponse(response)
+        self.assertEqual(data['id'], ext_product.id)
+        self.assertEqual(data['name'], ext_product.name)
+        self.assertEqual(data['brand'], ext_product.brand.id)
+        self.assertEqual(data['categories'], [category.id])
+        self.assertEqual(data['price'], str(ext_product.price))
+        self.assertEqual(data['order'], ext_product.order)
+        self.assertEqual(data['description'], ext_product.description)
+        # Attribute on model
+        self.assertEqual(data['is_priced_under_10'], True)
+        # Callable on model
+        self.assertEqual(data['get_first_category_id'], category.id)
+
+
+class InvalidViewTests(TestHelper):
+    def setUp(self):
+        BrandBackboneView.display_fields += ['invalid_field']
+
+    def tearDown(self):
+        BrandBackboneView.display_fields.remove('invalid_field')
+
+    def test_invalid_field_name_raises_attribute_error(self):
+        brand = self.create_brand()
+        url = reverse('backbone:tests_brand_detail', args=[brand.id])
+        try:
+            self.client.get(url)
+        except AttributeError, err:
+            self.assertEqual(str(err), "Invalid field: invalid_field")
